#!/usr/bin/env python
# coding=UTF-8


def _init():
    global _global_dict
    _global_dict = {}


def SetValue(name, value):
    _global_dict[name] = value


def GetValue(name):
    return _global_dict.get(name, 0)


def GetLen():
    return len(_global_dict)


def SetVehicleState(vehicle_state):
    SetValue('vehicle_state', vehicle_state)


def GetVehicleState():
    return GetValue('vehicle_state')


def SetControlMode(control_mode):
    SetValue('control_mode', control_mode)


def GetControlMode():
    return GetValue('control_mode')


def SetBatteryVoltage(battery_voltage):
    SetValue('battery_voltage', battery_voltage)


def GetBatteryVoltage():
    return GetValue('battery_voltage')


def SetErrorCode(error_code):
    SetValue('error_code', error_code)


def GetErrorCode():
    return GetValue('error_code')


def SetLinearVelocity(linear_velocity):
    SetValue('linear_velocity', linear_velocity)


def GetLinearVelocity():
    return GetValue('linear_velocity')


def SetAngularVelocity(angular_velocity):
    SetValue('angular_velocity', angular_velocity)


def GetAngularVelocity():
    return GetValue('angular_velocity')


def SetLateralVelocity(lateral_velocity):
    SetValue('lateral_velocity', lateral_velocity)


def GetLateralVelocity():
    return GetValue('lateral_velocity')


def SetSteeringAngle(steering_angle):
    SetValue('steering_angle', steering_angle)


def GetSteeringAngle():
    return GetValue('steering_angle')


def SetRpm1(rpm1):
    SetValue('rpm1', rpm1)


def GetRpm1():
    return GetValue('rpm1')


def SetRpm2(rpm2):
    SetValue('rpm2', rpm2)


def GetRpm2():
    return GetValue('rpm2')


def SetRpm3(rpm3):
    SetValue('rpm3', rpm3)


def GetRpm3():
    return GetValue('rpm3')


def SetRpm4(rpm4):
    SetValue('rpm4', rpm4)


def GetRpm4():
    return GetValue('rpm4')


def SetCurrent1(current1):
    SetValue('current1', current1)


def GetCurrent1():
    return GetValue('current1')


def SetCurrent2(current2):
    SetValue('current2', current2)


def GetCurrent2():
    return GetValue('current2')


def SetCurrent3(current3):
    SetValue('current3', current3)


def GetCurrent3():
    return GetValue('current3')


def SetCurrent4(current4):
    SetValue('current4', current4)


def GetCurrent4():
    return GetValue('current4')


def SetPulseCount1(pulse_count1):
    SetValue('pulse_count1', pulse_count1)


def GetPulseCount1():
    return GetValue('pulse_count1')


def SetPulseCount2(pulse_count2):
    SetValue('pulse_count2', pulse_count2)


def GetPulseCount2():
    return GetValue('pulse_count2')


def SetPulseCount3(pulse_count3):
    SetValue('pulse_count3', pulse_count3)


def GetPulseCount3():
    return GetValue('pulse_count3')


def SetPulseCount4(pulse_count4):
    SetValue('pulse_count4', pulse_count4)


def GetPulseCount4():
    return GetValue('pulse_count4')


def SetDriver1Voltage(driver1_voltage):
    SetValue('driver1_voltage', driver1_voltage)


def GetDriver1Voltage():
    return GetValue('driver1_voltage')


def SetDriver1Temp(driver1_temp):
    SetValue('driver1_temp', driver1_temp)


def GetDriver1Temp():
    return GetValue('driver1_temp')


def SetMotor1Temp(motor1_temp):
    SetValue('motor1_temp', motor1_temp)


def GetMotor1Temp():
    return GetValue('motor1_temp')


def SetDriver1State(driver1_state):
    SetValue('driver1_state', driver1_state)


def GetDriver1State():
    return GetValue('driver1_state')


def SetDriver2Voltage(driver2_voltage):
    SetValue('driver2_voltage', driver2_voltage)


def GetDriver2Voltage():
    return GetValue('driver2_voltage')


def SetDriver2Temp(driver2_temp):
    SetValue('driver2_temp', driver2_temp)


def GetDriver2Temp():
    return GetValue('driver2_temp')


def SetMotor2Temp(motor2_temp):
    SetValue('motor2_temp', motor2_temp)


def GetMotor2Temp():
    return GetValue('motor2_temp')


def SetDriver2State(driver2_state):
    SetValue('driver2_state', driver2_state)


def GetDriver2State():
    return GetValue('driver2_state')


def SetDriver3Voltage(driver3_voltage):
    SetValue('driver3_voltage', driver3_voltage)


def GetDriver3Voltage():
    return GetValue('driver3_voltage')


def SetDriver3Temp(driver3_temp):
    SetValue('driver_temp', driver3_temp)


def GetDriver3Temp():
    return GetValue('driver3_temp')


def SetMotor3Temp(motor3_temp):
    SetValue('motor3_temp', motor3_temp)


def GetMotor3Temp():
    return GetValue('motor3_temp')


def SetDriver3State(driver3_state):
    SetValue('driver3_state', driver3_state)


def GetDriver3State():
    return GetValue('driver3_state')


def SetDriver4Voltage(driver4_voltage):
    SetValue('driver4_voltage', driver4_voltage)


def GetDriver4Voltage():
    return GetValue('driver4_voltage')


def SetDriver4Temp(driver4_temp):
    SetValue('driver4_temp', driver4_temp)


def GetDriver4Temp():
    return GetValue('driver4_temp')


def SetMotor4Temp(motor4_temp):
    SetValue('motor4_temp', motor4_temp)


def GetMotor4Temp():
    return GetValue('motor4_temp')


def SetDriver4State(driver4_state):
    SetValue('driver_state', driver4_state)


def GetDriver4State():
    return GetValue('driver4_state')


def SetLightCmdCtrl(light_cmd_ctrl):
    SetValue('light_cmd_ctrl', light_cmd_ctrl)


def GetLightCmdCtrl():
    return GetValue('light_cmd_ctrl')


def SetFrontMode(front_mode):
    SetValue('front_mode', front_mode)


def GetFrontMode():
    return GetValue('front_mode')


def SetFrontCustom(front_custom):
    SetValue('front_custom', front_custom)


def GetFrontCustom():
    return GetValue('front_custom')


def SetRearMode(rear_mode):
    SetValue('rear_mode', rear_mode)


def GetRearMode():
    return GetValue('rear_mode')


def SetRearCustom(rear_custom):
    SetValue('rear_custom', rear_custom)


def GetRearCustom():
    return GetValue('rear_custom')


def SetControlHardwareVersion(control_hardware_version):
    SetValue('control_hardware_version', control_hardware_version)


def GetControlHardwareVersion():
    return GetValue('control_hardware_version')


def SetActuarorHardwareVersion(actuaror_hardware_version):
    SetValue('actuaror_hardware_version', actuaror_hardware_version)


def GetActuarorHardwareVersion():
    return GetValue('actuaror_hardware_version')


def SetControlSoftwareVersion(control_software_version):
    SetValue('control_software_version', control_software_version)


def GetControlSoftwareVersion():
    return GetValue('control_software_version')


def SetActuarorSoftwareVersion(actuaror_software_version):
    SetValue('actuaror_software_version', actuaror_software_version)


def GetActuarorSoftwareVersion():
    return GetValue('actuaror_software_version')


def SetLeftWheel(left_wheel):
    SetValue('left_wheel', left_wheel)


def GetLeftWheel():
    return GetValue('left_wheel')


def SetRightWheel(right_wheel):
    SetValue('right_wheel', right_wheel)


def GetRightWheel():
    return GetValue('right_wheel')


def SetSws(sws):
    SetValue('sws', sws)


def GetSws():
    return GetValue('sws')


def SetStickRightV(stick_right_v):
    SetValue('stick_right_v', stick_right_v)


def GetStickRightV():
    return GetValue('stick_right_v')


def SetStickRightH(stick_right_h):
    SetValue('stick_right_h', stick_right_h)


def GetStickRightH():
    return GetValue('stick_right_h')


def SetStickLeftV(stick_left_v):
    SetValue('stick_left_v', stick_left_v)


def GetStickLeftV():
    return GetValue('stick_left_v')


def SetStickLeftH(stick_left_h):
    SetValue('stick_left_h', stick_left_h)


def GetStickLeftH():
    return GetValue('stick_left_h')


def SetVarA(var_a):
    SetValue('var_a', var_a)


def GetVarA():
    return GetValue('var_a')


def SetBaseNeutral(base_neutral):
    SetValue('base_neutral', base_neutral)


def GetBaseNeutral():
    return GetValue('base_neutral')


class CanMsgLen(object):
    CAN_MSG_LEN = 53


class CanIDV2(object):
    # // control group: 0x1
    MOTION_COMMAND_ID = 0X111
    LIGHT_COMMAND_ID = 0X121
    BRAKING_COMMAND_ID = 0X131
    SET_MOTION_MODE_ID = 0X141

    # // state feedback group: 0x2
    SYSTEM_STATE_ID = 0X211
    MOTION_STATE_ID = 0X221
    LIGHT_STATE_ID = 0X231
    RC_STATE_ID = 0X241

    # //get catuator hight speed state
    ACTUATOR1_HS_STATE_ID = 0X251
    ACTUATOR2_HS_STATE_ID = 0X252
    ACTUATOR3_HS_STATE_ID = 0X253
    ACTUATOR4_HS_STATE_ID = 0X254
    ACTUATOR5_HS_STATE_ID = 0X255
    ACTUATOR6_HS_STATE_ID = 0X256
    ACTUATOR7_HS_STATE_ID = 0X257
    ACTUATOR8_HS_STATE_ID = 0X258

    # //get actuator low speed state
    ACTUATOR1_LS_STATE_ID = 0X261
    ACTUATOR2_LS_STATE_ID = 0X262
    ACTUATOR3_LS_STATE_ID = 0X263
    ACTUATOR4_LS_STATE_ID = 0X264
    ACTUATOR5_LS_STATE_ID = 0X265
    ACTUATOR6_LS_STATE_ID = 0X266
    ACTUATOR7_LS_STATE_ID = 0X267
    ACTUATOR8_LS_STATE_ID = 0X268

    MOTOR_ANGLE_INFO_ID = 0X271
    MOTOR_SPEED_INFO_ID = 0X281
    CURRENT_CTRL_MODE_ID = 0X291

    # // sensor data group: 0x3
    ODOMETRY_ID = 0X311
    IMU_ACCEL_ID = 0X321
    IMU_GYRO_ID = 0X322
    IMU_EULER_ID = 0X323
    SAFETY_BUMPER_ID = 0X331

    BMS_BASIC_ID = 0X361
    BMS_EXTENDED_ID = 0X362

    # // query/config group: 0x4
    VERSION_REQUEST_ID = 0X411
    VERSION_RESPONSE_ID = 0X41A
    CTRL_MODE_CONFIG_ID = 0X421
    STEER_NEUTRAL_REQUEST_ID = 0X431
    STEER_NEUTRAL_RESPONSE_ID = 0X43A
    STATE_RESET_CONFIG_ID = 0X441


class CanIDV1(object):
    # CAN: control group

    MOTION_COMMAND_ID = 0x130
    LIGHT_COMMAND_ID = 0x140
    VALUE_SET_COMMAND_ID = 0x210

    # CAN: state feedback group
    MOTION_STATE_ID = 0x131
    LIGHT_STATE_ID = 0x141
    SYSTEM_STATE_ID = 0x151

    VALUE_SET_STATE_ID = 0x211

    ACTUATOR1_STATE_ID = 0x200
    ACTUATOR2_STATE_ID = 0x201
    ACTUATOR3_STATE_ID = 0x202
    ACTUATOR4_STATE_ID = 0x203



import pyagxrobots.UGVConfigMsg as UGVBaseMsg


class MotionCommandMessage(object):
    def GetLinearVelocity(self):
        return float(UGVBaseMsg.GetLinearVelocity())

    def GetAngularVelocity(self):
        # // only valid for differential drivering
        return float(UGVBaseMsg.GetAngularVelocity())

    def GetLateralVelocity(self):
        return float(UGVBaseMsg.GetLateralVelocity())

    def GetSteeringAngle(self):
        # // only valid for ackermann steering
        return float(UGVBaseMsg.GetSteeringAngle())


class LightCommandMessage(object):
    def GetLightCmdCtrl(self):
        return UGVBaseMsg.GetLightCmdCtrl()

    def GetFrontMode(self):
        return UGVBaseMsg.GetFrontMode()

    def GetFrontCustom(self):
        return UGVBaseMsg.GetFrontCustom()

    def GetRearMode(self):
        return UGVBaseMsg.GetRearMode()

    def GetRearCustom(self):
        return UGVBaseMsg.GetRearCustom()


class SystemStateMessage(object):
    def GetVehicleState(self):
        return UGVBaseMsg.GetVehicleState()

    def GetControlMode(self):
        return UGVBaseMsg.GetControlMode()

    def GetBatteryVoltage(self):
        return float(UGVBaseMsg.GetBatteryVoltage())

    def GetErrorCode(self):
        return UGVBaseMsg.GetErrorCode()


class RcStateMessage(object):
    def GetVarA(self):
        return UGVBaseMsg.GetVarA()

    def GetSws(self):
        return UGVBaseMsg.GetSws()

    def GetStickRightV(self):
        return UGVBaseMsg.GetStickRightV()

    def GetStickRightH(self):
        return UGVBaseMsg.GetStickRightH()

    def GetStickLeftV(self):
        return UGVBaseMsg.GetStickLeftV()

    def GetStickLeftH(self):
        return UGVBaseMsg.GetStickLeftH()


class OdometryMessage(object):
    def GetLeftWheel(self):
        return float(UGVBaseMsg.GetLeftWheel())

    def GetRightWheel(self):
        return float(UGVBaseMsg.GetRightWheel())


class ActuatorStateMessageV1(object):
    def __init__(self, motro_id=0):
        self.motro_id = motro_id

    def current(self):
        if self.motro_id == 1:
            return UGVBaseMsg.GetCurrent1()
        elif self.motro_id == 2:
            return UGVBaseMsg.GetCurrent2()
        elif self.motro_id == 3:
            return UGVBaseMsg.GetCurrent3()
        elif self.motro_id == 4:
            return UGVBaseMsg.GetCurrent4()

    def rpm(self):
        if self.motro_id == 1:
            return UGVBaseMsg.GetRpm1()
        elif self.motro_id == 2:
            return UGVBaseMsg.GetRpm2()
        elif self.motro_id == 3:
            return UGVBaseMsg.GetRpm3()
        elif self.motro_id == 4:
            return UGVBaseMsg.GetRpm4()

    def driver_temp(self):
        if self.motro_id == 1:
            return UGVBaseMsg.GetDriver1Temp()
        elif self.motro_id == 2:
            return UGVBaseMsg.GetDriver2Temp()
        elif self.motro_id == 3:
            return UGVBaseMsg.GetDriver3Temp()
        elif self.motro_id == 4:
            return UGVBaseMsg.GetDriver4Temp()

    def motor_temp(self):
        if self.motro_id == 1:
            return UGVBaseMsg.GetMotor1Temp()
        elif self.motro_id == 2:
            return UGVBaseMsg.GetMotor2Temp()
        elif self.motro_id == 3:
            return UGVBaseMsg.GetMotor3Temp()
        elif self.motro_id == 4:
            return UGVBaseMsg.GetMotor4Temp()


class ActuatorStateMessageV2(object):
    def __init__(self, motro_id=0):
        self.motro_id = motro_id

    def rpm(self):
        if self.motro_id == 1:
            return UGVBaseMsg.GetRpm1()
        elif self.motro_id == 2:
            return UGVBaseMsg.GetRpm2()
        elif self.motro_id == 3:
            return UGVBaseMsg.GetRpm3()
        elif self.motro_id == 4:
            return UGVBaseMsg.GetRpm4()

    def current(self):
        if self.motro_id == 1:
            return UGVBaseMsg.GetCurrent1()
        elif self.motro_id == 2:
            return UGVBaseMsg.GetCurrent2()
        elif self.motro_id == 3:
            return UGVBaseMsg.GetCurrent3()
        elif self.motro_id == 4:
            return UGVBaseMsg.GetCurrent4()

    def pulse_count(self):
        if self.motro_id == 1:
            return UGVBaseMsg.GetPulseCount1()
        elif self.motro_id == 2:
            return UGVBaseMsg.GetPulseCount2()
        elif self.motro_id == 3:
            return UGVBaseMsg.GetPulseCount3()
        elif self.motro_id == 4:
            return UGVBaseMsg.GetPulseCount4()

    def driver_voltage(self):
        if self.motro_id == 1:
            return UGVBaseMsg.GetDriver1Voltage()
        elif self.motro_id == 2:
            return UGVBaseMsg.GetDriver2Voltage()
        elif self.motro_id == 3:
            return UGVBaseMsg.GetDriver3Voltage()
        elif self.motro_id == 4:
            return UGVBaseMsg.GetDriver4Voltage()

    def driver_temp(self):
        if self.motro_id == 1:
            return UGVBaseMsg.GetDriver1Temp()
        elif self.motro_id == 2:
            return UGVBaseMsg.GetDriver2Temp()
        elif self.motro_id == 3:
            return UGVBaseMsg.GetDriver3Temp()
        elif self.motro_id == 4:
            return UGVBaseMsg.GetDriver4Temp()

    def motor_temp(self):
        if self.motro_id == 1:
            return UGVBaseMsg.GetMotor1Temp()
        elif self.motro_id == 2:
            return UGVBaseMsg.GetMotor2Temp()
        elif self.motro_id == 3:
            return UGVBaseMsg.GetMotor3Temp()
        elif self.motro_id == 4:
            return UGVBaseMsg.GetMotor4Temp()

    def driver_state(self):
        if self.motro_id == 1:
            return UGVBaseMsg.GetDriver1State()
        elif self.motro_id == 2:
            return UGVBaseMsg.GetDriver2State()
        elif self.motro_id == 3:
            return UGVBaseMsg.GetDriver3State()
        elif self.motro_id == 4:
            return UGVBaseMsg.GetDriver4State()


class GetRobotStae(MotionCommandMessage,
                   LightCommandMessage,
                   SystemStateMessage,
                   RcStateMessage,
                   OdometryMessage):
    pass



#!/usr/bin/env python3
# coding=UTF-8
import sys
import os

from wrapt.wrappers import transient_function_wrapper

import pyagxrobots.UGVConfigMsg as UGVBaseMsg
import pyagxrobots.agxrobots as agxrobots
from pyagxrobots.agxbase import ActuatorStateMessageV1, GetRobotStae
from pyagxrobots.agxbase import ActuatorStateMessageV2


class BunkerBase(GetRobotStae):
    def __init__(self) :
        self.basetype='bunker'
        self.motronum= 2
        self.bunkerbase=agxrobots.UGV()
        self.baseversion=self.bunkerbase.base_version
        self.max_linear=3.0
        self.max_angular=2.5235
        self.max_lateral=0.0
        self.min_linear=-self.max_linear
        self.min_angular=-self.max_angular
        self.min_lateral=-self.max_lateral
        if self.baseversion==1:
            self.BunkerActuator1=ActuatorStateMessageV1(motro_id=1)
            self.BunkerActuator2=ActuatorStateMessageV1(motro_id=2)
        elif self.baseversion==2:
            self.BunkerActuator1=ActuatorStateMessageV2(motro_id=1)
            self.BunkerActuator2=ActuatorStateMessageV2(motro_id=2)

    def  SetMotionCommand(self,linear_vel,angular_vel):

        if linear_vel>self.max_linear:
            linear_vel=self.max_linear
        elif linear_vel<self.min_linear:
            linear_vel=self.min_linear
        if angular_vel>self.max_angular:
            angular_vel=self.max_angular
        elif angular_vel<self.min_angular:
            angular_vel=self.min_angular
        self.bunkerbase.SetMotionCommand(linear_vel,angular_vel,0,0)
  
        # self.version=   
    def EnableCAN(self):
        if self.baseversion==2:
            self.bunkerbase.EnableCANCtrl()   
       
    
    
   
class HunterBase(GetRobotStae):
    def __init__(self) :
        self.basetype='hunter'
        self.motronum= 2
        self.bunterbase=agxrobots.UGV()
        self.baseversion=self.bunterbase.base_version
        self.max_linear=1.5
        self.max_angular=0.444
        self.max_lateral=0.0
        self.min_linear=-self.max_linear
        self.min_angular=-self.max_angular
        self.min_lateral=-self.max_lateral
        if self.baseversion==1:
            self.HunterActuator1=ActuatorStateMessageV1(motro_id=1)
            self.HunterActuator2=ActuatorStateMessageV1(motro_id=2)
        elif self.baseversion==2:
            self.HunterActuator1=ActuatorStateMessageV2(motro_id=1)
            self.HunterActuator2=ActuatorStateMessageV2(motro_id=2)
    def  SetMotionCommand(self,linear_vel,angular_vel):
        if self.baseversion==1:
            if linear_vel>self.max_linear:
                linear_vel=self.max_linear
            elif linear_vel<self.min_linear:
                linear_vel=self.min_linear
            if angular_vel>self.max_angular:
                angular_vel=self.max_angular
            elif angular_vel<self.min_angular:
                angular_vel=self.min_angular       
        self.bunterbase.SetMotionCommand(linear_vel,0,0,angular_vel)

    def EnableCAN(self):
        if self.baseversion==2:
            self.bunterbase.EnableCANCtrl()   



class RangerBase(GetRobotStae):
    def __init__(self) :
        self.basetype='ranger'
        self.motronum= 4
        self.rangerbase=agxrobots.UGV()
        self.baseversion=self.rangerbase.base_version
        if self.baseversion==1:
            self.RangerActuator1=ActuatorStateMessageV1(motro_id=1)
            self.RangerActuator2=ActuatorStateMessageV1(motro_id=2)
            self.RangerActuator3=ActuatorStateMessageV1(motro_id=3)
            self.RangerActuator4=ActuatorStateMessageV2(motro_id=4)
        elif self.baseversion==2:
            self.RangerActuator1=ActuatorStateMessageV2(motro_id=1)
            self.RangerActuator2=ActuatorStateMessageV2(motro_id=2)
            self.RangerActuator3=ActuatorStateMessageV2(motro_id=3)
            self.RangerActuator4=ActuatorStateMessageV2(motro_id=4)  

    def  SetMotionCommand(self,linear_vel=0.0,lateral_vel=0.0,angular_vel=0.0,steer_angle=0.0):
        
        self.rangerbase.SetMotionCommand(linear_vel,angular_vel,lateral_vel,steer_angle)
    def SetLightCommand(self,f_mode,f_value,r_mode,r_value):
        self.rangerbase.SetLightCommand(f_mode,f_value,r_mode,r_value)   

       # self.version= 
    def EnableCAN(self):
        if self.baseversion==2:
            self.rangerbase.EnableCANCtrl()   

class ScoutBase(GetRobotStae):
    def __init__(self):
        self.basetype='scout'
        self.motronum= 4
        self.scoutbase=agxrobots.UGV()
        self.baseversion=self.scoutbase.base_version
        self.max_linear=1.5
        self.max_angular=0.5235
        self.max_lateral=0.0
        self.min_linear=-self.max_linear
        self.min_angular=-self.max_angular
        self.min_lateral=-self.max_lateral
        if self.baseversion==1:
            self.ScoutActuator1=ActuatorStateMessageV1(motro_id=1)
            self.ScoutActuator2=ActuatorStateMessageV1(motro_id=2)
            self.ScoutActuator3=ActuatorStateMessageV1(motro_id=3)
            self.ScoutActuator4=ActuatorStateMessageV2(motro_id=4)
        elif self.baseversion==2:
            self.ScoutActuator1=ActuatorStateMessageV2(motro_id=1)
            self.ScoutActuator2=ActuatorStateMessageV2(motro_id=2)
            self.ScoutActuator3=ActuatorStateMessageV2(motro_id=3)
            self.ScoutActuator4=ActuatorStateMessageV2(motro_id=4)        

    def  SetMotionCommand(self,linear_vel=0.0,angular_vel=0.0):
        if self.baseversion==2:
            if linear_vel>self.max_linear:
                linear_vel=self.max_linear
            elif linear_vel<self.min_linear:
                linear_vel=self.min_linear
            if angular_vel>self.max_angular:
                angular_vel=self.max_angular
            elif angular_vel<self.min_angular:
                angular_vel=self.min_angular
        self.scoutbase.SetMotionCommand(linear_vel,angular_vel,0,0)
    def SetLightCommand(self,f_mode,f_value,r_mode,r_value):
        self.scoutbase.SetLightCommand(f_mode,f_value,r_mode,r_value)
        # self.version= 

    def EnableCAN(self):
        if self.baseversion==2:
            self.scoutbase.EnableCANCtrl()   
        

class ScoutMiniBase(GetRobotStae):
    def __init__(self):
        self.basetype='scoutMini'
        self.motronum= 4
        self.scoutminibase=agxrobots.UGV()
        self.baseversion=self.scoutminibase.base_version
        self.max_linear=3.0
        self.max_angular=2.5235
        self.max_lateral=2.0
        self.min_linear=-self.max_linear
        self.min_angular=-self.max_angular
        self.min_lateral=-self.max_lateral
        if self.baseversion==1:
            self.ScoutMiniActuator1=ActuatorStateMessageV1(motro_id=1)
            self.ScoutMiniActuator2=ActuatorStateMessageV1(motro_id=2)
            self.ScoutMiniActuator3=ActuatorStateMessageV1(motro_id=3)
            self.ScoutMiniActuator4=ActuatorStateMessageV2(motro_id=4)
        elif self.baseversion==2:
            self.ScoutMiniActuator1=ActuatorStateMessageV2(motro_id=1)
            self.ScoutMiniActuator2=ActuatorStateMessageV2(motro_id=2)
            self.ScoutMiniActuator3=ActuatorStateMessageV2(motro_id=3)
            self.ScoutMiniActuator4=ActuatorStateMessageV2(motro_id=4)

    
    def  SetMotionCommand(self,linear_vel=0.0,angular_vel=0.0,lateral_velocity=0.0):
        if self.baseversion==1:
            if linear_vel<0:
                linear_vel=0
            elif linear_vel>100:
                linear_vel=100
        elif self.baseversion==2:
            if linear_vel>self.max_linear:
                linear_vel=self.max_linear
            elif linear_vel<self.min_linear:
                linear_vel=self.min_linear
            if angular_vel>self.max_angular:
                angular_vel=self.max_angular
            elif angular_vel<self.min_angular:
                angular_vel=self.min_angular
            if lateral_velocity>self.max_lateral:
                lateral_velocity=self.max_lateral
            elif lateral_velocity<self.min_lateral:
                lateral_velocity=self.min_lateral
        self.scoutminibase.SetMotionCommand(linear_vel,angular_vel,lateral_velocity,0)
    def SetLightCommand(self,f_mode,f_value,r_mode,r_value):
        self.scoutminibase.SetLightCommand(f_mode,f_value,r_mode,r_value)
    def EnableCAN(self):
        if self.baseversion==2:
            self.scoutminibase.EnableCANCtrl()
      # self.version= 


class TracerBase(GetRobotStae):
    def __init__(self):
        self.basetype='tracer'
        self.motronum= 2
        self.tracerbase=agxrobots.UGV()
        self.baseversion=self.tracerbase.base_version
        if self.baseversion==1:
            self.TracerActuator1=ActuatorStateMessageV1(motro_id=1)
            self.TracerActuator2=ActuatorStateMessageV1(motro_id=2)
        elif self.baseversion==2:
            self.TracerActuator1=ActuatorStateMessageV2(motro_id=1)
            self.TracerActuator2=ActuatorStateMessageV2(motro_id=2)

    def  SetMotionCommand(self,linear_vel=0.0,angular_vel=0.0):
        self.tracerbase.SetMotionCommand(linear_vel,angular_vel,0,0)
    def SetLightCommand(self,f_mode,f_value):
        self.tracerbase.SetLightCommand(f_mode,f_value)

    def EnableCAN(self):
        if self.baseversion==2:
            self.tracerbase.EnableCANCtrl()
        # self.version= 
    # TracerAuator1=ActuatorStateMessageV2(motro_id=1)
    # TracerAuator2=ActuatorStateMessageV2(motro_id=2)
    


#!/usr/bin/env python3
# coding=UTF-8

import sys
import os
import asyncio
import can
import time
import threading
import ctypes
import struct
import math
from can import Message
from typing import ByteString, Set, Type
from sqlite3.dbapi2 import Date

import pyagxrobots.UGVConfigMsg as UGVBaseMsg

# Type of current operating system
# posix： Linux
# nt: Windows
# java:

os_is_nt = os.name == 'nt'
# Linux and Mac are both part of the posix standard
# posix: Portable API for Unix-like operating systems
os_is_posix = os.name == 'posix'


class CanMsgsGet:
    def __init__(self, capacity=1024 * 4):
        self.size = 0
        self.rear = 0
        self.front = 0
        self.array = capacity

    def CanMsgsProcessV1(self, msg):
        if (msg.arbitration_id == UGVBaseMsg.CanIDV1.SYSTEM_STATE_ID):
            vehicle_state = int(msg.data[0])
            UGVBaseMsg.SetVehicleState(vehicle_state)
            control_mode = msg.data[1]
            UGVBaseMsg.SetControlMode(control_mode)
            battery_voltage = float((msg.data[2] & 0xff) << 8
                                    | msg.data[3]) / 10
            UGVBaseMsg.SetBatteryVoltage(battery_voltage)
            error_code = (msg.data[4] << 8) | msg.data[5]
            UGVBaseMsg.SetErrorCode(error_code)

            # print(
            #     'vehicle_state:%s control_mode:%s battery_voltage:%s error_code:%s  count_num:%s'
            #     % (vehicle_state, control_mode, battery_voltage, error_code,
            #        count_num))
        elif (msg.arbitration_id == UGVBaseMsg.CanIDV1.MOTION_STATE_ID):
            linear_velocity_get = ctypes.c_int16((msg.data[0] & 0xff) << 8
                                                 | msg.data[1])

            linear_velocity = float(linear_velocity_get.value / 1000)
            # print(int(linear_velocity))
            UGVBaseMsg.SetLinearVelocity(linear_velocity)
            angular_velocity_get = ctypes.c_int16((msg.data[2] & 0xff) << 8
                                                  | msg.data[3])
            angular_velocity = float(angular_velocity_get.value / 1000)
            UGVBaseMsg.SetAngularVelocity(angular_velocity)
            lateral_velocity_get = ctypes.c_int16((msg.data[4] & 0xff) << 8
                                                  | msg.data[5])
            lateral_velocity = float(lateral_velocity_get.value / 1000)
            UGVBaseMsg.SetLateralVelocity(lateral_velocity)

            # print(msg)

            # print(msg)
            # print(
            #     'linear_velocity:%s angular_velocity:%s lateral_velocity:%s steering_angle:%s '
            #     % (linear_velocity, angular_velocity, lateral_velocity,
            #        steering_angle))
        elif (msg.arbitration_id == UGVBaseMsg.CanIDV1.LIGHT_STATE_ID):
            light_cmd_ctrl = (msg.data[0])
            UGVBaseMsg.SetLightCmdCtrl(light_cmd_ctrl)
            front_mode = (msg.data[1])
            UGVBaseMsg.SetFrontMode(front_mode)
            front_custom = (msg.data[2])
            UGVBaseMsg.SetFrontCustom(front_custom)
            rear_mode = (msg.data[3])
            UGVBaseMsg.SetRearMode(rear_mode)
            rear_custom = (msg.data[4])
            UGVBaseMsg.SetRearCustom(rear_custom)

        elif (msg.arbitration_id == UGVBaseMsg.CanIDV1.VALUE_SET_STATE_ID):
            if (msg.data[0] == 0xaa):
                base_neutral = True
            else:
                base_neutral = False
            UGVBaseMsg.SetBaseNeutral(base_neutral)

        elif(msg.arbitration_id == UGVBaseMsg.CanIDV1.ACTUATOR1_STATE_ID):
            current1 = float((msg.data[0] << 8 | msg.data[1])/10.0)
            UGVBaseMsg.SetCurrent1(current1)
            rpm1 = ctypes.c_int16(msg.data[2] << 8 | msg.data[3]).value
            UGVBaseMsg.SetRpm1(rpm1)
            driver1_temp = msg.date[4]
            UGVBaseMsg.SetDriver1State(driver1_temp)
            motor1_temp = msg.data[5]
            UGVBaseMsg.SetMotor1Temp(motor1_temp)

        elif(msg.arbitration_id == UGVBaseMsg.CanIDV1.ACTUATOR2_STATE_ID):
            current2 = float((msg.data[0] << 8 | msg.data[1])/10.0)
            UGVBaseMsg.SetCurrent2(current2)
            rpm2 = ctypes.c_int16(msg.data[2] << 8 | msg.data[3]).value
            UGVBaseMsg.SetRpm2(rpm2)
            driver2_temp = msg.date[4]
            UGVBaseMsg.SetDriver2State(driver2_temp)
            motor2_temp = msg.data[5]
            UGVBaseMsg.SetMotor2Temp(motor2_temp)

        elif(msg.arbitration_id == UGVBaseMsg.CanIDV1.ACTUATOR3_STATE_ID):
            current3 = float((msg.data[0] << 8 | msg.data[1])/10.0)
            UGVBaseMsg.SetCurrent3(current3)
            rpm3 = ctypes.c_int16(msg.data[2] << 8 | msg.data[3]).value
            UGVBaseMsg.SetRpm3(rpm3)
            driver3_temp = msg.date[4]
            UGVBaseMsg.SetDriver3State(driver3_temp)
            motor3_temp = msg.data[5]
            UGVBaseMsg.SetMotor3Temp(motor3_temp)

        elif(msg.arbitration_id == UGVBaseMsg.CanIDV1.ACTUATOR4_STATE_ID):
            current4 = float((msg.data[0] << 8 | msg.data[1])/10.0)
            UGVBaseMsg.SetCurrent4(current4)
            rpm4 = ctypes.c_int16(msg.data[2] << 8 | msg.data[3]).value
            UGVBaseMsg.SetRpm4(rpm4)
            driver4_temp = msg.date[4]
            UGVBaseMsg.SetDriver4State(driver4_temp)
            motor4_temp = msg.data[5]
            UGVBaseMsg.SetMotor4Temp(motor4_temp)

    def CanMsgsProcessV2(self, msg):
        """
        Process Can bus data according to ID
        """

        msg = msg
        if (msg.arbitration_id == UGVBaseMsg.CanIDV2.SYSTEM_STATE_ID):
            vehicle_state = int(msg.data[0])
            UGVBaseMsg.SetVehicleState(vehicle_state)
            control_mode = msg.data[1]
            UGVBaseMsg.SetControlMode(control_mode)
            battery_voltage = float((msg.data[2] & 0xff) << 8
                                    | msg.data[3]) / 10
            UGVBaseMsg.SetBatteryVoltage(battery_voltage)
            error_code = msg.data[5]
            UGVBaseMsg.SetErrorCode(error_code)
            count_num = msg.data[7]

            # print(
            #     'vehicle_state:%s control_mode:%s battery_voltage:%s error_code:%s  count_num:%s'
            #     % (vehicle_state, control_mode, battery_voltage, error_code,
            #        count_num))
        elif (msg.arbitration_id == UGVBaseMsg.CanIDV2.MOTION_STATE_ID):
            linear_velocity_get = ctypes.c_int16((msg.data[0] & 0xff) << 8
                                                 | msg.data[1])

            linear_velocity = float(linear_velocity_get.value / 1000)
            # print(int(linear_velocity))
            UGVBaseMsg.SetLinearVelocity(linear_velocity)
            angular_velocity_get = ctypes.c_int16((msg.data[2] & 0xff) << 8
                                                  | msg.data[3])
            angular_velocity = float(angular_velocity_get.value / 1000)
            UGVBaseMsg.SetAngularVelocity(angular_velocity)
            lateral_velocity_get = ctypes.c_int16((msg.data[4] & 0xff) << 8
                                                  | msg.data[5])
            lateral_velocity = float(lateral_velocity_get.value / 1000)
            UGVBaseMsg.SetLateralVelocity(lateral_velocity)
            steering_angle_get = ctypes.c_int16((msg.data[6] & 0xff) << 8
                                                | msg.data[7])
            steering_angle = float(steering_angle_get.value / 1000)
            UGVBaseMsg.SetSteeringAngle(steering_angle)
            # print(msg)

            # print(msg)
            # print(
            #     'linear_velocity:%s angular_velocity:%s lateral_velocity:%s steering_angle:%s '
            #     % (linear_velocity, angular_velocity, lateral_velocity,
            #        steering_angle))

        elif (msg.arbitration_id == UGVBaseMsg.CanIDV2.ACTUATOR1_HS_STATE_ID):
            rpm1 = int((msg.data[0] & 0xff) << 8 | msg.data[1])
            UGVBaseMsg.SetRpm1(rpm1)
            current1 = float((msg.data[2] & 0xff) << 8 | msg.data[3]) * 0.1
            UGVBaseMsg.SetCurrent1(current1)
            pulse_count1 = int((msg.data[4] & 0xff) << 24
                               | (msg.data[5] & 0xff) << 16
                               | (msg.data[6] & 0xff) << 8
                               | msg.data[7])
            UGVBaseMsg.SetPulseCount1(pulse_count1)
            # print('rpm1:%s current1:%s pulse_count1:%s ' %
            #       (rpm1, current1, pulse_count1))

        elif (msg.arbitration_id == UGVBaseMsg.CanIDV2.ACTUATOR2_HS_STATE_ID):
            rpm2 = int((msg.data[0] & 0xff) << 8 | msg.data[1])
            UGVBaseMsg.SetRpm2(rpm2)
            current2 = float((msg.data[2] & 0xff) << 8 | msg.data[3]) * 0.1
            UGVBaseMsg.SetCurrent2(current2)
            pulse_count2 = int((msg.data[4] & 0xff) << 24
                               | (msg.data[5] & 0xff) << 16
                               | (msg.data[6] & 0xff) << 8
                               | msg.data[7])
            UGVBaseMsg.SetPulseCount2(pulse_count2)
            # print('rpm2:%s current2:%s pulse_count2:%s ' %
            #       (rpm2, current2, pulse_count2))

        elif (msg.arbitration_id == UGVBaseMsg.CanIDV2.ACTUATOR3_HS_STATE_ID):
            rpm3 = int((msg.data[0] & 0xff) << 8 | msg.data[1])
            UGVBaseMsg.SetRpm3(rpm3)
            current3 = float((msg.data[2] & 0xff) << 8 | msg.data[3]) * 0.1
            UGVBaseMsg.SetCurrent3(current3)
            pulse_count3 = int((msg.data[4] & 0xff) << 24
                               | (msg.data[5] & 0xff) << 16
                               | (msg.data[6] & 0xff) << 8
                               | msg.data[7])
            UGVBaseMsg.SetPulseCount3(pulse_count3)
            # print('rpm3:%s current3:%s pulse_count3:%s ' %
            #       (rpm3, current3, pulse_count3))

        elif (msg.arbitration_id == UGVBaseMsg.CanIDV2.ACTUATOR4_HS_STATE_ID):
            rpm4 = int((msg.data[0] & 0xff) << 8 | msg.data[1])
            UGVBaseMsg.SetRpm4(rpm4)
            current4 = float((msg.data[2] & 0xff) << 8 | msg.data[3]) * 0.1
            UGVBaseMsg.SetCurrent4(current4)
            pulse_count4 = int((msg.data[4] & 0xff) << 24
                               | (msg.data[5] & 0xff) << 16
                               | (msg.data[6] & 0xff) << 8
                               | msg.data[7])
            UGVBaseMsg.SetPulseCount4(pulse_count4)

            # print('rpm4:%s current4:%s pulse_count4:%s ' %
            #       (rpm4, current4, pulse_count4))

        elif (msg.arbitration_id == UGVBaseMsg.CanIDV2.ACTUATOR1_LS_STATE_ID):
            driver1_voltage = float((msg.data[0] & 0xff) << 8
                                    | msg.data[1]) * 0.1
            UGVBaseMsg.SetDriver1Voltage(driver1_voltage)
            driver1_temp = int((msg.data[2] & 0xff) << 8 | msg.data[3])
            UGVBaseMsg.SetDriver1Temp(driver1_temp)
            motor1_temp = msg.data[4]
            UGVBaseMsg.SetMotor1Temp(motor1_temp)
            driver1_state = msg.data[5]
            UGVBaseMsg.SetDriver1State(driver1_state)
            # print(
            #     'driver_voltage1:%s driver_temp1:%s motor_temp%s driver_state1:%s '
            #     % (driver1_voltage, driver1_temp, motor1_temp, driver1_state))

        elif (msg.arbitration_id == UGVBaseMsg.CanIDV2.ACTUATOR2_LS_STATE_ID):
            driver2_voltage = float((msg.data[0] & 0xff) << 8
                                    | msg.data[1]) * 0.1
            UGVBaseMsg.SetDriver2Voltage(driver2_voltage)
            driver2_temp = int((msg.data[2] & 0xff) << 8 | msg.data[3])
            UGVBaseMsg.SetDriver2Temp(driver2_temp)
            motor2_temp = msg.data[4]
            UGVBaseMsg.SetMotor2Temp(motor2_temp)
            driver2_state = msg.data[5]
            UGVBaseMsg.SetDriver2State(driver2_state)
            # print(
            #     'driver_voltage2:%s driver_temp2:%s  motor_temp2:%s driver_state2:%s '
            #     % (driver2_voltage, driver2_temp, motor2_temp, driver2_state))
        elif (msg.arbitration_id == UGVBaseMsg.CanIDV2.ACTUATOR3_LS_STATE_ID):
            driver3_voltage = float((msg.data[0] & 0xff) << 8
                                    | msg.data[1]) * 0.1
            UGVBaseMsg.SetDriver3Voltage(driver3_voltage)
            driver3_temp = int((msg.data[2] & 0xff) << 8 | msg.data[3])
            UGVBaseMsg.SetDriver3Temp(driver3_temp)
            motor3_temp = msg.data[4]
            UGVBaseMsg.SetMotor3Temp(motor3_temp)
            driver3_state = msg.data[5]
            UGVBaseMsg.SetDriver3State(driver3_state)
            # print(
            #     'driver_voltage3:%s driver_temp3:%s  motor_temp3:%s driver_state3:%s '
            #     % (driver3_voltage, driver3_temp, motor3_temp, driver3_state))
        elif (msg.arbitration_id == UGVBaseMsg.CanIDV2.ACTUATOR4_LS_STATE_ID):
            driver4_voltage = float((msg.data[0] & 0xff) << 8
                                    | msg.data[1]) * 0.1
            UGVBaseMsg.SetDriver4Voltage(driver4_voltage)
            driver4_temp = int((msg.data[2] & 0xff) << 8 | msg.data[3])
            UGVBaseMsg.SetDriver4Temp(driver4_temp)
            motor4_temp = msg.data[4]
            UGVBaseMsg.SetMotor4Temp(motor4_temp)
            driver4_state = msg.data[5]
            UGVBaseMsg.SetDriver4State(driver4_state)
            # print(
            #     'driver_voltage4:%s driver_temp4:%s  motor_temp:%s driver_state4:%s '
            #     % (driver4_voltage, driver4_temp, motor4_temp, driver4_state))

        elif (msg.arbitration_id == UGVBaseMsg.CanIDV2.LIGHT_STATE_ID):
            light_cmd_ctrl = (msg.data[0])
            UGVBaseMsg.SetLightCmdCtrl(light_cmd_ctrl)
            front_mode = (msg.data[1])
            UGVBaseMsg.SetFrontMode(front_mode)
            front_custom = (msg.data[2])
            UGVBaseMsg.SetFrontCustom(front_custom)
            rear_mode = (msg.data[3])
            UGVBaseMsg.SetRearMode(rear_mode)
            rear_custom = (msg.data[4])
            UGVBaseMsg.SetRearCustom(rear_custom)

            # print(
            #     'enable_cmd_ctrl:%s front_mode:%s  front_custom:%s rear_mode:%s  rear_custom:%s'
            #     % (light_cmd_ctrl, front_mode, front_custom, rear_mode,
            #        rear_custom))
        elif (msg.arbitration_id == UGVBaseMsg.CanIDV2.VERSION_RESPONSE_ID):
            control_hardware_version = int((msg.data[0] & 0xff) << 8
                                           | msg.data[1])
            UGVBaseMsg.SetControlHardwareVersion(control_hardware_version)
            actuaror_hardware_version = int((msg.data[2] & 0xff) << 8
                                            | msg.data[3])
            UGVBaseMsg.SetActuarorHardwareVersion(actuaror_hardware_version)
            control_software_version = int((msg.data[4] & 0xff) << 8
                                           | msg.data[5])
            UGVBaseMsg.SetControlSoftwareVersion(control_software_version)
            actuaror_software_version = int((msg.data[6] & 0xff) << 8
                                            | msg.data[7])
            UGVBaseMsg.SetActuarorSoftwareVersion(actuaror_software_version)
            # print(
            #     'control_hardware_version: %s actuaror_hardware_version: %s' %
            #     (control_hardware_version, actuaror_hardware_version))
            # print('control_software_version:%s actuaror_software_version:%s ' %
            #       (control_software_version, actuaror_software_version))

        elif (msg.arbitration_id == UGVBaseMsg.CanIDV2.ODOMETRY_ID):
            left_wheel_get = ctypes.c_int((msg.data[0] & 0xff) << 24
                                          | (msg.data[1] & 0xff) << 16
                                          | (msg.data[2] & 0xff) << 8
                                          | msg.data[3])
            left_wheel = left_wheel_get.value
            UGVBaseMsg.SetLeftWheel(left_wheel)
            right_wheel_get = ctypes.c_int((msg.data[4] & 0xff) << 24
                                           | (msg.data[5] & 0xff) << 16
                                           | (msg.data[6] & 0xff) << 8
                                           | msg.data[7])
            # print('left_wheel: %s right_wheel: %s ' %
            #       (left_wheel, right_wheel))
            right_wheel = right_wheel_get.value
            UGVBaseMsg.SetRightWheel(right_wheel)
            # print(msg)
        elif (msg.arbitration_id == UGVBaseMsg.CanIDV2.RC_STATE_ID):
            sws = msg.data[0]
            UGVBaseMsg.SetSws(sws)
            stick_right_v = msg.data[1]
            UGVBaseMsg.SetStickRightV(stick_right_v)
            stick_right_h = msg.data[2]
            UGVBaseMsg.SetStickRightH(stick_right_h)
            stick_left_v = msg.data[3]
            UGVBaseMsg.SetStickLeftV(stick_left_v)
            stick_left_h = msg.data[4]
            UGVBaseMsg.SetStickLeftH(stick_left_h)
            var_a = msg.data[5]
            UGVBaseMsg.SetVarA(var_a)


class DeviceCan:
    """

    """

    def __init__(self, bustype=None, channel=None, bitrate=None):
        UGVBaseMsg._init()
        if bitrate is None:
            self.bitrate = 500000
        else:
            self.bitrate = bitrate

        if bustype is None:
            self.bustype = "socketcan"
        else:
            self.bustype = bustype

        if channel is None:
            self.channel = "can0"
        else:
            default_channel = channel
            self.channel = default_channel
        self.base_verson = self.GetBaseVersion()

        # with self.canport as server:
        with can.interface.Bus(bustype=self.bustype,
                               channel=self.channel,
                               bitrate=self.bitrate) as client:
            # stop_event = threading.Event()
            t_receive = threading.Thread(target=self.EnableAsynsCan)
            t_receive.start()
            # self.EnableAsynsCan(client)

            try:
                if self.base_verson==1:
                    pass
                elif self.base_verson==2:
                    # while (UGVBaseMsg.GetLen() < UGVBaseMsg.CanMsgLen.CAN_MSG_LEN):
                    #     self.msg = Message(
                    #         arbitration_id=UGVBaseMsg.CanIDV2.VERSION_REQUEST_ID,
                    #         data=[0x01],
                    #         is_extended_id=False)
                    #     self.CanSend(self.msg)
                        time.sleep(0.1)

            except (KeyboardInterrupt, SystemExit):
                pass
            # stop_event.set()
            time.sleep(0.5)

    def GetBaseVersion(self):
        self.base_verson=0
        self.canport = can.interface.Bus(bustype=self.bustype,
                                         channel=self.channel,
                                         bitrate=self.bitrate)
        while (self.base_verson == 0):
            msg = self.canport.recv(0.01)
            if(msg.arbitration_id == 0x151):
                self.base_verson = 1
                return self.base_verson
            elif(msg.arbitration_id == 0x241):
                self.base_verson = 2
                return self.base_verson
            else:
                self.base_verson = 0
                

    async def CanReceive(self, bus, stop_event):
        """The loop for receiving."""
        print("Start receiving messages")
        SlectMsg = CanMsgsGet()
        while not stop_event.is_set():

            self.rx_msg = await bus.recv(0.002)
            if self.rx_msg is not None:
                if(self.base_verson == 1):
                    SlectMsg.CanMsgsProcessV2(self.rx_msg)
                    # print("rx: {}".format(self.rx_msg))
                elif(self.base_verson == 2):
                    SlectMsg.CanMsgsProcessV2(self.rx_msg)
                    # print("rx: {}".format(self.rx_msg))

        await bus.recv()
        print("Stopped receiving messages")
        if stop_event.is_set() == False:
            stop_event.set()

    def CanGet(self):
        self.canport = can.ThreadSafeBus(interface=self.bustype,
                                         channel=self.channel,
                                         bitrate=self.bitrate)

        msg = self.canport.recv(0.002)
        print(msg)
        return msg

    def EnableAsynsCan(self):

        LOOP = asyncio.new_event_loop()
        asyncio.set_event_loop(LOOP)
        # Run until main coroutine finishes
        LOOP.run_until_complete(self.AsynsCan())

    def ListenerMessage(self, msg):
        SlectMsg = CanMsgsGet()
        if (self.base_verson == 1):
            SlectMsg.CanMsgsProcessV1(msg)
        elif (self.base_verson == 2):
            SlectMsg.CanMsgsProcessV2(msg)

    async def AsynsCan(self):
        bus = can.Bus(interface=self.bustype,
                      channel=self.channel,
                      bitrate=self.bitrate)

        reader = can.BufferedReader()
        logger = can.Logger('logfile.asc')
        listeners = [
            self.ListenerMessage,
            reader,  # AsyncBufferedReader() listener
            logger  # Regular Listener object
        ]
        loop = asyncio.get_event_loop()
        notifier = can.Notifier(bus, listeners, loop=loop)
        while 1:
            msg = reader.get_message()
            await asyncio.sleep(0.2)

        notifier.stop()

    def CanDataUpdate(self):
        msg = self.CanGet()
        SlectMsg = CanMsgsGet()
        if msg is None:
            print('time occerred,no message')
        elif(self.base_verson == 1):
            SlectMsg.CanMsgsProcessV1(msg)
        elif(self.base_verson == 2):
            SlectMsg.CanMsgsProcessV2(msg)

    def CanSend(self, msg):

        self.canport = can.ThreadSafeBus(interface=self.bustype,
                                         channel=self.channel,
                                         bitrate=self.bitrate)
        with self.canport as bus:
            try:
                bus.send(msg)
            except can.CanError:
                print("Message NOT sent")


class UGV:
    """
        EnableCANCtrl()
        SendVersionRequest()
        SendErrorClearByte()
        EnableLightCtrl()
        DisableLightCtrl()
        LightFrontMode()
        SendLinerVelocity()
        SendAngularVelocity()

        GetLightMode()        
        GetSysVersion()
        GetLeftWheelOdem()
        GetRightWheelOdem()
        GetLinerVelocity()
        GetAngularVelocity()
        GetErrorCode()

    """

    def __init__(self, *device_args, **device_kwargs):
        self.device = None
        can_device_init_fn = DeviceCan.__init__
        args_names = can_device_init_fn.__code__.co_varnames[:
                                                             can_device_init_fn
                                                             .__code__.
                                                             co_argcount]
        args_dict = dict(zip(args_names, device_args))
        args_dict.update(device_kwargs)

        self.device = DeviceCan(**args_dict)
        self.base_version = self.device.base_verson
    UGVBaseMsg._init()

    def SendMsg(self, msg):
        self.device.CanSend(msg)

    def GetMsg(self):
        return self.device.CanGet()

    def SendVersionRequest(self):
        """
        Version Information Request
        """
        self.msg = Message(arbitration_id=UGVBaseMsg.CanIDV2.VERSION_REQUEST_ID,
                           data=[0x01],
                           is_extended_id=False)
        self.SendMsg(self.msg)

    def EnableCANCtrl(self):
        """
        控制底盘处于Can控制模式，需将遥控SWB拨到最上方
        Control the chassis in Can control mode, you need to dial the remote control SWB to the top
        """

        self.msg = Message(arbitration_id=UGVBaseMsg.CanIDV2.CTRL_MODE_CONFIG_ID,
                           data=[0x01],
                           is_extended_id=False)
        self.SendMsg(self.msg)

    def SetMotionCommand(self,
                         linear_vel: float = 0.0,
                         angular_vel: float = 0.0,
                         lateral_vel: float = 0.0,
                         steering_angle: float = 0.0):

        if self.base_version == 1:
            if abs(angular_vel)>abs(steering_angle):
                angular=angular_vel
            else: angular = steering_angle
            msg = Message(arbitration_id=UGVBaseMsg.CanIDV1.MOTION_COMMAND_ID,
                          data=[
                              0x01,
                              0,
                              (int(linear_vel )& 0xff),
                              (int(angular) & 0xff),
                              (int(lateral_vel) & 0xff),
                              0,
                              0,
                              0,
                          ],
                          is_extended_id=False)
            self.SendMsg(msg)

        elif self.base_version == 2:
            linear_cmd = linear_vel * 1000
            angular_cmd = angular_vel * 1000
            lateral_cmd = lateral_vel * 1000
            steering_cmd = steering_angle * 1000
            msg = Message(arbitration_id=UGVBaseMsg.CanIDV2.MOTION_COMMAND_ID,
                          data=[
                              int(linear_cmd) >> 8 & 0xff,
                              (int(linear_cmd) & 0x00ff),
                              int(angular_cmd) >> 8 & 0xff,
                              (int(angular_cmd) & 0x00ff),
                              int(lateral_cmd) >> 8 & 0xff,
                              (int(lateral_cmd) & 0x00ff),
                              int(steering_cmd) >> 8 & 0xff,
                              (int(steering_cmd) & 0x00ff)
                          ],
                          is_extended_id=False)
            self.SendMsg(msg)

    def SetLightCommand(self, front_mode: int = 0, front_custom_value: int = 0, rear_mode: int = 0, rear_custom_value: int = 0):
        if self.base_version==1:
            msg = Message(arbitration_id=UGVBaseMsg.CanIDV1.LIGHT_COMMAND_ID,
                        data=[
                            0x01,
                            front_mode,
                            front_custom_value,
                            rear_mode,
                            rear_custom_value,
                            0,
                            0,
                            0
                        ],
                        is_extended_id=False)
        elif self.base_version==2:
            msg = Message(arbitration_id=UGVBaseMsg.CanIDV2.LIGHT_COMMAND_ID,
                        data=[
                            0x01,
                            front_mode,
                            front_custom_value,
                            rear_mode,
                            rear_custom_value,
                            0,
                            0,
                            0
                        ],
                        is_extended_id=False)
        self.SendMsg(msg)



from pyagxrobots.agxrobots import UGV
from pyagxrobots.agxrobots import CanMsgsGet
from pyagxrobots.agxrobots import DeviceCan
from pyagxrobots import UGVConfigMsg
from pyagxrobots import pysdkugv
from pyagxrobots import agxbase
__all__ = [
    'UGV',
    'CanMsgsGet',
    'DeviceCan',
    'UGVConfigMsg',
    'pysdkugv',
    'agxbase',
]







from setuptools import setup, find_packages

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name='pyagxrobots',
    version='0.8.4',
    author='agilexrobotics',
    author_email='support@agilex.ai',
    description='A small example package for agxrobots',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/agilexrobotics/',
    project_urls={
        'Bug Tracker': 'https://github.com/agilexrobotics/',
    },
    classifiers=[
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'License :: OSI Approved :: MIT License',
        
    ],

    package_dir={'': 'src'},
    packages=find_packages(where='src'),
    install_requires=['python-can>=3.3.4'],
    python_requires='>=3.6',
)



#!/usr/bin/env python3
# coding=UTF-8
import pyagxrobots
import time
scoutmini=pyagxrobots.pysdkugv.ScoutMiniBase()
scoutmini.EnableCAN()
num=5
while num>0:
    
    scoutmini.SetMotionCommand(linear_vel=0.1)
    print('rpm')
    print(scoutmini.ScoutMiniActuator2.rpm())
    print('GetLinearVelocity')
    print(scoutmini.GetLinearVelocity())
    time.sleep(0.3)
    num-=1
